@@ -77,6 +77,7 @@ class Collate:
         gene_id_col = "Gene symbol" if colname != "refseq_protein_accession" else "Accession of closest sequence"
         
         d = reftab[reftab[colname] == row[1][gene_id_col]]['enhanced_subclass'].values[0]
+        
         if d in self.NONRTM:
             return self.RTM
         elif d in self.MACROLIDES:
@@ -266,7 +267,7 @@ class MduCollate(Collate):
         fh.setFormatter(formatter)
         self.logger.addHandler(ch) 
         self.logger.addHandler(fh)
-
+        self.sop = args.sop
         self.mduqc = args.qc
         self.db = args.db
         self.partials = args.partials
@@ -304,19 +305,6 @@ class MduCollate(Collate):
             gene = gene.replace("bla", "")
         return gene
 
-    def get_passed_isolates(self, qc_tab):
-        """
-        generate lists of isolates that passed QC and need AMR, failed QC and should have AMR and all other isolates
-        """        
-        failed = list(
-            qc_tab[qc_tab["TEST_QC"] == False]["ISOLATE"]
-        )  # isolates that failed qc and should have amr
-        passed = list(
-            qc_tab[qc_tab["TEST_QC"] == True]["ISOLATE"]
-        )  
-
-        return (passed, failed)
-
 
     # Carbapenemase to be reported for all cases
     # Carbapenemase (MBL) all in all HOWEVER if blaL1 should  not be reported in Stenotrophomonas maltophilia
@@ -347,8 +335,180 @@ class MduCollate(Collate):
         else:
             return "GENRL_AMR_NEG1"
 
+    def assign_itemcode(self,mduid, reg):
+        self.logger.info(f"Checking for item code")
+        m = reg.match(mduid)
+        try:
+            itemcode = m.group('itemcode') if m.group('itemcode') else ''
+        except AttributeError:
+            itemcode = ''
+        return itemcode
 
-    def reporting_logic(self, row, species, neg_code = True):
+    def assign_mduid(self, mduid, reg):
+        self.logger.info(f"Extracting MDU sample ID")
+        m = reg.match(mduid)
+        try:
+            mduid = m.group('id')
+        except AttributeError:
+            mduid = mduid.split('/')[-1]
+        return mduid
+
+    def _ampicillin_res_sal(self, col, gene):
+
+        if col in [ 'Beta-lactamase (not ESBL or carbapenemase)', 'Beta-lactamase (narrow-spectrum)']:
+            return gene
+        return ''
+    
+    def _cefo_esbl_res_sal(self, col, gene):
+
+        if col == 'ESBL':
+            return gene
+        return ''
+
+    def _cefo_ampc_res_sal(self, col, gene):
+
+        if col == 'ESBL (AmpC type)':
+            return gene
+        return ''
+    
+    def _carbapenem_res_salmo(self, col, gene):
+
+        if col in ["Carbapenemase", "Carbapenemase (MBL)", "Carbapenemase OXA-51 family"]:
+            return gene
+        return ''
+
+    def _azi_res_salmo(self, col, gene):
+
+        if col == 'Macrolide, lincosamide & streptogramin resistance':
+            return gene
+        return ''
+
+    def _gentamicin_res_salm(self, col, gene):
+        if 'Gentamicin' in col or col == "Other aminoglycoside resistance (non-RMT)":
+            return gene
+        return ''
+    
+    def _tetra_res_salmo(self, col, gene):
+
+        if col == 'Tetracycline':
+            return gene
+        return ''
+
+    def _cipro_res_salmo(self, col, gene):
+
+        if 'Quinolone' in col:
+            return gene
+        return ''
+
+    def _sulf_res_salmo(self, col, gene):
+        
+        if col == "Sulfonamide":
+            return gene
+        return ''
+    
+    def _trimet_res_salmo(self, col, gene):
+
+        if col == "Trimethoprim":
+            return gene
+        return ''
+
+    def _rmt_res_salmo(self, col, gene):
+
+        if col == "Aminoglycosides (Ribosomal methyltransferase)":
+            return gene
+        return ''
+
+    def _colistin_res_salmo(self, col, gene):
+        if col == 'Colistin':
+            return gene
+        return ''
+    
+    def reporting_logic_salmonella(self, row):
+
+        # Ampicillin - ResMech	Ampicillin - Interpretation	
+        # Cefotaxime (ESBL) - ResMech	Cefotaxime (ESBL) - Interpretation	
+        # Cefotaxime (AmpC) - ResMech	Cefotaxime (AmpC) - Interpretation	
+        # Tetracycline - ResMech	Tetracycline - Interpretation	
+        # Gentamycin - ResMech	Gentamycin - Interpretation	
+        # Sulfathiazole - ResMech	Sulfathiazole - Interpretation	
+        # Trimethoprim - ResMech	Trimethoprim - Interpretation	
+        # Ciprofloxacin - ResMech	Ciprofloxacin - Interpretation	
+        # Azithromycin - ResMech	Azithromycin - Interpretation
+
+        mduidreg = re.compile(r'(?P<id>[0-9]{4}-[0-9]{5,6})-?(?P<itemcode>.{1,2})?')
+        
+        all_genes = self.get_all_genes(row)
+        all_genes = [a for a in all_genes if a != row[1]['Isolate'] and a != '']
+        # print(all_genes)
+        
+        isodict = row[1].to_dict()
+        item_code = self.assign_itemcode(row[1]['Isolate'], mduidreg)
+        md = self.assign_mduid(row[1]['Isolate'], mduidreg)
+
+        abx = {
+            "Ampicillin" : self._ampicillin_res_sal,
+            "Cefotaxime (ESBL)":self._cefo_esbl_res_sal,
+            "Cefotaxime (AmpC)":self._cefo_ampc_res_sal,
+            "Carbapenem" :self._carbapenem_res_salmo,
+            "Azithromycin":self._azi_res_salmo,
+            "Gentamicin":self._gentamicin_res_salm,
+            "Tetracycline":self._tetra_res_salmo,
+            "Ciprofloxacin":self._cipro_res_salmo,
+            "Sulfathiazole":self._sulf_res_salmo,
+            "Trimethoprim":self._trimet_res_salmo,
+            "Aminoglycosides (Ribosomal methyltransferase)": self._rmt_res_salmo,
+            "Colistin":self._colistin_res_salmo
+        }
+
+        
+        tmp_results = {
+            "Ampicillin":[],	
+            "Cefotaxime (ESBL)":[],
+            "Cefotaxime (AmpC)":[],
+            "Tetracycline":[],
+            "Gentamicin":[],
+            "Sulfathiazole":[],
+            "Trimethoprim":[],
+            "Ciprofloxacin":[],
+            "Azithromycin":[],
+            "Carbapenem":[],
+            "Aminoglycosides (Ribosomal methyltransferase)":[],"Colistin":[], "Other":[]
+        }
+        # group drugs
+        for ab in abx:
+            for i in isodict:
+                if i != "Isolate":
+                    # print(isodict[i])
+                    g = abx[ab](col = i, gene = isodict[i])
+                    # print(g)
+                    if g != '':
+                        gene_list = g.split(',')
+                        for gene in gene_list:
+                            tmp_results[ab].append(gene)
+                            all_genes.remove(gene)
+        # interprete results
+        tmp_results['Other'] = all_genes
+        results = {'Isolate': row[1]['Isolate'], 'MDU Sample ID':md, 'Item code': item_code}
+
+        for res in tmp_results:
+            results[f"{res} - ResMech"] = ';'.join(tmp_results[res])                
+            if res == 'Ciprofloxacin':
+                if tmp_results[res] == []:
+                    results[f"{res} - Interpretation"] = 'Susceptible'
+                elif len(tmp_results[res]) == 1:
+                    results[f"{res} - Interpretation"] = 'Intermediate'
+                else:
+                    results[f"{res} - Interpretation"] = 'Resistant'
+            elif res not in ["Aminoglycosides (Ribosomal methyltransferase)","Colistin", "Other"]:
+                if tmp_results[res] == []:
+                    results[f"{res} - Interpretation"] = 'Susceptible'
+                else:
+                    results[f"{res} - Interpretation"] = 'Resistant'
+        
+        return results
+
+            
+    def reporting_logic_general(self, row, species, neg_code = True):
         # get all genes found
         all_genes = self.get_all_genes(row)
         isodict = row[1].to_dict()
@@ -388,38 +548,39 @@ class MduCollate(Collate):
         genes_not_reported = []  # genes found but not reportable
         for i in isodict:
             genes = []
-            if isinstance(isodict[i], str):
-                genes = isodict[i].split(',')
-            if genes != []: # for each bin we do things to genes
-                if i in reportable:
-                    if i in non_caveat_reportable:
-                        genes_reported.extend(genes)
-                    elif i == "Carbapenemase (MBL)" and species != "Stenotrophomonas maltophilia":
-                        genes_reported.extend(genes)
-                    elif i == "Carbapenemase (MBL)" and species == "Stenotrophomonas maltophilia":
-                         # if species is "Stenotrophomonas maltophilia" don't report blaL1
-                        genes_reported.extend([g for g in genes if not g.startswith("blaL1")])
-                        genes_not_reported.extend([g for g in genes if g.startswith("blaL1")])
-                    elif i == "Carbapenemase (OXA-51 family)" and species not in abacter_excluded:
-                        genes_reported.extend(genes)
-                    elif i in ["ESBL","ESBL (AmpC type)"] and genus in ["Salmonella"]: 
-                        genes_reported.extend(genes)
-                    elif i in ["ESBL","ESBL (AmpC type)"] and genus in ["Shigella"]: 
-                        genes_reported.extend([g for g in genes if "blaEC" not in g])
-                        genes_not_reported.extend([g for g in genes if "blaEC" in g]) # don't report blaEC for shigella
-                    elif i == "Oxazolidinone & phenicol resistance":
-                        if species in ["Staphylococcus aureus","Staphylococcus argenteus"] or genus == "Enterococcus":
+            if i != 'Isolate':
+                if isinstance(isodict[i], str):
+                    genes = isodict[i].split(',')
+                if genes != []: # for each bin we do things to genes
+                    if i in reportable:
+                        if i in non_caveat_reportable:
                             genes_reported.extend(genes)
+                        elif i == "Carbapenemase (MBL)" and species != "Stenotrophomonas maltophilia":
+                            genes_reported.extend(genes)
+                        elif i == "Carbapenemase (MBL)" and species == "Stenotrophomonas maltophilia":
+                            # if species is "Stenotrophomonas maltophilia" don't report blaL1
+                            genes_reported.extend([g for g in genes if not g.startswith("blaL1")])
+                            genes_not_reported.extend([g for g in genes if g.startswith("blaL1")])
+                        elif i == "Carbapenemase (OXA-51 family)" and species not in abacter_excluded:
+                            genes_reported.extend(genes)
+                        elif i in ["ESBL","ESBL (AmpC type)"] and genus in ["Salmonella"]: 
+                            genes_reported.extend(genes)
+                        elif i in ["ESBL","ESBL (AmpC type)"] and genus in ["Shigella"]: 
+                            genes_reported.extend([g for g in genes if "blaEC" not in g])
+                            genes_not_reported.extend([g for g in genes if "blaEC" in g]) # don't report blaEC for shigella
+                        elif i == "Oxazolidinone & phenicol resistance":
+                            if species in ["Staphylococcus aureus","Staphylococcus argenteus"] or genus == "Enterococcus":
+                                genes_reported.extend(genes)
+                            else:
+                                genes_not_reported.extend(genes)
+                        elif i == "Vancomycin":
+                            genes_reported.extend([g for g in genes if van_match.match(g)])
+                            genes_not_reported.extend([g for g in genes if not van_match.match(g)])
+                        elif i == "Methicillin":
+                            genes_reported.extend([g for g in genes if mec_match.match(g)])
+                            genes_not_reported.extend([g for g in genes if not mec_match.match(g)])
                         else:
                             genes_not_reported.extend(genes)
-                    elif i == "Vancomycin":
-                        genes_reported.extend([g for g in genes if van_match.match(g)])
-                        genes_not_reported.extend([g for g in genes if not van_match.match(g)])
-                    elif i == "Methicillin":
-                        genes_reported.extend([g for g in genes if mec_match.match(g)])
-                        genes_not_reported.extend([g for g in genes if not mec_match.match(g)])
-                    else:
-                        genes_not_reported.extend(genes)
 
                 else:
                     genes_not_reported.extend(genes)
@@ -432,26 +593,30 @@ class MduCollate(Collate):
         self.logger.info(f"{row[1]['Isolate']} has {len(genes_reported)} reportable genes.")
         return genes_reported, genes_not_reported
 
+    def mdu_reporting_salmonella(self, match):
 
-    def assign_itemcode(self,mduid, reg):
-        self.logger.info(f"Checking for item code")
-        m = reg.match(mduid)
-        try:
-            itemcode = m.group('itemcode') if m.group('itemcode') else ''
-        except AttributeError:
-            itemcode = ''
-        return itemcode
+        self.logger.info(f"Applying MDU business logic for interpretation of  Salmonella AST")
+        cols = ["Isolate", "MDU Sample ID", "Item code", "Ampicillin - ResMech", "Ampicillin - Interpretation","Cefotaxime (ESBL) - ResMech","Cefotaxime (ESBL) - Interpretation","Cefotaxime (AmpC) - ResMech","Cefotaxime (AmpC) - Interpretation",
+            "Tetracycline - ResMech","Tetracycline - Interpretation","Gentamicin - ResMech","Gentamicin - Interpretation","Sulfathiazole - ResMech","Sulfathiazole - Interpretation","Trimethoprim - ResMech","Trimethoprim - Interpretation",
+            "Ciprofloxacin - ResMech","Ciprofloxacin - Interpretation","Azithromycin - ResMech","Azithromycin - Interpretation","Aminoglycosides (Ribosomal methyltransferase) - ResMech","Colistin - ResMech", "Other - ResMech"]
+        # select passed Salmonella
+        qc = self.mdu_qc_tab()
+        qc = qc[(qc['SPECIES_OBS'] == 'Salmonella enterica') & (qc['TEST_QC'] == 'PASS')]
 
-    def assign_mduid(self, mduid, reg):
-        self.logger.info(f"Extracting MDU sample ID")
-        m = reg.match(mduid)
-        try:
-            mduid = m.group('id')
-        except AttributeError:
-            mduid = mduid.split('/')[-1]
-        return mduid
+        df = pandas.read_csv(match, sep = '\t')
+        df = df[df['Isolate'].isin(list(qc['ISOLATE']))]
+        result_df = pandas.DataFrame()
+        df = df.fillna('')
+        for row in df.iterrows():
+            isolate_results = self.reporting_logic_salmonella(row = row)
+            tmpdf = pandas.DataFrame(isolate_results, index = [0])
+            if result_df.empty:
+                result_df = tmpdf
+            else:
+                result_df = result_df.append(tmpdf)
+        return result_df[cols]
 
-    def mdu_reporting(self, match, neg_code = True):
+    def mdu_reporting_general(self, match, neg_code = True):
 
         self.logger.info(f"Applying MDU business logic {'matches' if neg_code else 'partials'}.")
         mduidreg = re.compile(r'(?P<id>[0-9]{4}-[0-9]{5,6})-?(?P<itemcode>.{1,2})?')
@@ -468,12 +633,15 @@ class MduCollate(Collate):
             obs_species = qcdf["SPECIES_OBS"].values[0]
            
             species = obs_species if obs_species == exp_species else exp_species
-            genes_reported, genes_not_reported = self.reporting_logic(
+            genes_reported, genes_not_reported = self.reporting_logic_general(
                 row=row, species=species, neg_code=neg_code
             )
+            
             # strip bla
             genes_not_reported = [self.strip_bla(g) for g in genes_not_reported]
+            genes_not_reported = [g for g in genes_not_reported if g != isolate]
             genes_reported = [self.strip_bla(g) for g in genes_reported]
+            genes_reported = [g for g in genes_reported if g != isolate]
             d["Resistance genes (alleles) detected"] = ",".join(genes_reported)
             d["Resistance genes (alleles) det (non-rpt)"] = ",".join(genes_not_reported)
             if qcdf["TEST_QC"].values[0] == 'FAIL': # species not needed for MDU LIMS upload
@@ -492,24 +660,33 @@ class MduCollate(Collate):
         return reporting_df.reindex(labels = ['Item code','Resistance genes (alleles) detected','Resistance genes (alleles) det (non-rpt)','Species_obs', 'Species_exp', 'db_version'], axis = 'columns')
 
     
-    def save_spreadsheet(
+    def save_spreadsheet_general(
         self,
         passed_match,
         passed_partials,
         
     ):
+        self.logger.info(f"Saving MMS118.")
         writer = pandas.ExcelWriter(f"{self.runid}_MMS118.xlsx", engine="xlsxwriter")
         passed_match.to_excel(writer, sheet_name="MMS118")
         passed_partials.to_excel(writer, sheet_name="Passed QC partial")
         writer.close()
 
+    def save_spreadsheet_interpreted(self, results):
+        self.logger.info(f"Saving MMS184")
+        writer = pandas.ExcelWriter(f"{self.runid}_MMS184.xlsx", engine = "xlsxwriter")
+        results.to_excel(writer, sheet_name = "MMS184")
+        writer.close()
+
     def run(self):
+        if self.sop == 'general':
+            passed_match_df = self.mdu_reporting_general(match=self.match)
+            passed_partials_df = self.mdu_reporting_general(match = self.partials)
+            self.save_spreadsheet_general(
+                passed_match_df,
+                passed_partials_df
+            )
+        elif self.sop == 'salmonella':
 
-        passed_match_df = self.mdu_reporting(match=self.match)
-        passed_partials_df = self.mdu_reporting(match = self.partials, neg_code=False)
-        self.save_spreadsheet(
-            passed_match_df,
-            passed_partials_df,
-
-        )
-
+            salmo_df = self.mdu_reporting_salmonella(match = self.match)
+            self.save_spreadsheet_interpreted(results = salmo_df)
